##################### PSQF 6243 Example 4b using R ##############################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=140, digits=6, scipen=4, show.signif.stars=FALSE)

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import excel XLSX data as table

# Masks describe function in psych package
if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get correlations with table of p-values

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("supernova")) install.packages("supernova")
library(supernova) # To get sums of squares table

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("lmhelprs")) install.packages("lmhelprs")
library(lmhelprs) # To get F-test and change in R2 in GLMs

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files using code below

# Clear environment (run as needed)
#rm(list=ls())

################################################################################
#####              BEGIN FUNCTIONS TO AUTOMATE CALCULATIONS                #####
#####       JUST RUN THIS CODE AS IS; SHOULD NOT NEED TO CHANGE IT         #####
################################################################################

# This SummaryCI function saves a summary of a model object, and then adds 
#  confidence intervals (default is 95%) to the fixed effects table 
SummaryCI = function(model, level=.95){
  modelsummary = summary(model)        # Save model summary
  CIs = confint(model, level=level)    # Save fixed effects CIs
  result = data.frame(modelsummary$coefficients, CIs) # Combine into one table and rename
  colnames(result) = c("Estimate","StdErr","t-value","p-value","LowerCI","UpperCI")
  return(result) # Print result
}

# This FixedEffectSizes function saves a summary of an lm model object, and then adds
#   columns to the summary for Partial D (Cohen's D), Partial R, Semi-Partial R, 
#   Partial R2, and Semi-Partial R2 for all fixed effects
FixedEffectSizes = function(model){
  fullModelSummary = summary(model)  # Save model summary and compute effect sizes
  pD = (2*fullModelSummary$coefficients[,"t value"])/sqrt(model$df.residual)  # Cohen's d
  pR =    fullModelSummary$coefficients[,"t value"]/
    sqrt(fullModelSummary$coefficients[,"t value"]^2+model$df.residual)       # Partial R
  sR =    fullModelSummary$coefficients[,"t value"]*
    sqrt((1-summary(model)$r.squared)/model$df.residual)                      # Semi-Partial R
  result = data.frame(fullModelSummary$coefficients, pD, pR, sR, pR^2, sR^2)  # Merge to one table
  result = subset(result, select=-c(Std..Error,t.value)) # Drop unneeded columns
  colnames(result) = c("Estimate","p-value","Partial-d","Partial-r",  # Rename columns
                       "SemiPartial-r","Partial-R2","SemiPartial-R2")
  return(result) # Print result
}

# This glhtSummaryCI function saves a summary of a glht object, and then adds 
#  confidence intervals (default is 95%) to the summary for the linear combinations 
glhtSummaryCI = function(glhtObject, level=.95){
  glhtSummary = summary(glhtObject, test=adjusted("none")) # Save unadjusted p-values
  CIs = confint(glhtObject, level=level, calpha=univariate_calpha()) # Get CIs
  result = data.frame(Estimate=glhtSummary$test$coefficients, 
                      StdErr=glhtSummary$test$sigma,
                      pvalue=glhtSummary$test$pvalues, CIs$confint) # Merge to one table
  result = subset(result, select=-c(Estimate.1))
  colnames(result) = c("Estimate","StdErr","p-value","LowerCI","UpperCI")
  return(result) # Print result
}

# This glhtEffectSizes function saves a summary of a glht object, and then adds
#   columns to that summary for Partial D (Cohen's D), Partial R, Semi-Partial R, 
#   Partial R2, and Semi-Partial R2 for all linear combinations
glhtEffectSizes = function(glhtObject, modelObject, level=.95){
  glhtSummary = summary(glhtObject, test=adjusted("none")) # save unadjusted p-values
  pD = (2*glhtSummary$test$tstat)/sqrt(modelObject$df.residual)
  pR = glhtSummary$test$tstat/sqrt(glhtSummary$test$tstat^2+modelObject$df.residual)
  sR = glhtSummary$test$tstat*sqrt((1-summary(modelObject)$r.squared)/modelObject$df.residual)
  result = data.frame(Estimate=glhtSummary$test$coefficients, pvalue=glhtSummary$test$pvalues, 
                      pD, pR, sR, pR^2, sR^2)
  colnames(result) = c("Estimate","p-value","Partial-d","Partial-r",  # Rename columns
                       "SemiPartial-r","Partial-R2","SemiPartial-R2")
  return(result) # Print result
}

# This R2changeF function prints the F-test and change in R2 for a model comparison
#   (as provided by the hierarchical_lm function in the lmhelprs package),
#   as well as partial R2 and semi-partial R2 (which is equal to R2 change here)
R2changeF = function(ReducedModel, FullModel, PredName){
  FullModelTable = supernova(FullModel) # Get total sums of squares for computations
  SStotal = FullModelTable[["tbl"]][["SS"]][length(FullModelTable[["tbl"]][["SS"]])]  
  R2F = hierarchical_lm(ReducedModel, FullModel) # Get values for F-test and R2 change
  SSeffect = R2F[2,7]    # Save effect sums of squares (as reduction in SS from full model)
  SSresidual = R2F[2,5]  # Save residual sums of squares from full model
  pr2 = SSeffect/(SSeffect+SSresidual)  # Compute Partial R2
  sr2 = SSeffect/SStotal                # Compute Semi-Partial R2
  cat(paste("F-Test and R2 Change for", PredName, "Slopes \n")) # Add better title
  colnames(R2F) = c("AdjR2","R2","R2change","DFden","RSS", "DFnum","SSeffect",
                    "Fvalue","pvalue")  # Better names
  result = data.frame(R2F$R2, R2F$R2change, R2F$DFnum, R2F$DFden, R2F$Fvalue, R2F$pvalue, 
                      pr2, sr2) # Re-order, add effect sizes computed above
  colnames(result) = c("R2-total","R2-change","DF-num","DF-den","F-value", 
                       "p-value","Partial-R2","SemiPartial-R2") # Rename for easier reading
  return(result[2,]) # Only print second row (first is no longer needed)
}


################################################################################
#####            BEGIN EXAMPLE 4b DATA IMPORT AND MANIPULATION             #####
################################################################################

# Set working directory (to import and export files to)
# Paste in the folder address where "Example4b_Data.xlsx" is saved in quotes
setwd("C:/Dropbox/24_PSQF6243/PSQF6243_Example4b")

# ImportExample4b_Data.xlsx data from working directory -- path = file name
Example4b = read_excel(path="Example4b_Data.xlsx", sheet="Example4b") 
# Convert to data frame to use for analysis
Example4b = as.data.frame(Example4b)

# Center quantitative predictors near their means
Example4b$age85=Example4b$age-85
Example4b$grip9=Example4b$grip-9

# Create 2 indicator-dummy-coded binary predictors for 3 dementia groups
Example4b$demNF=NA; Example4b$demNC=NA  # Create 2 new empty variables 
Example4b$demNF[which(Example4b$demgroup==1)]=0  # Replace each for none group
Example4b$demNC[which(Example4b$demgroup==1)]=0
Example4b$demNF[which(Example4b$demgroup==2)]=1  # Replace each for future group
Example4b$demNC[which(Example4b$demgroup==2)]=0
Example4b$demNF[which(Example4b$demgroup==3)]=0  # Replace each for current group
Example4b$demNC[which(Example4b$demgroup==3)]=1
# demNF: None=0 vs Future=1
# demNC: None=0 vs Current=1 

# Label variables used below (add descriptive titles) using comments instead
#   because variable labels are not allowed in many analysis packages
# Format: name: Descriptive Variable Label
#age85=     "age85: Age in Years (0=85)"
#grip9=     "grip9: Grip Strength in Pounds (0=9)"
#sexMW=     "sexMW: Sex (0=M, 1=W)"
#demNF=     "demNF: Dementia Contrast for None=0 vs Future=1"
#demNC=     "demNC: Dementia Contrast for None=0 vs Current=1"
#cognition= "cognition: Cognition Outcome"
#demgroup=  "demgroup: Dementia Group 1N 2F 3C"

# Select cases complete on all variables to be used
Example4b = Example4b[complete.cases(Example4b[ , 
            c("cognition","age","grip","sexMW","demgroup")]),]

# Open external file to save results to
txtStart(file="Example4b_R_Output.txt")


################################################################################
#####           UNIVARIATE DESCRIPTION AND BIVARIATE ANALYSES              #####
################################################################################

print("R Descriptive Statistics for Quantitative Variables")
psych::describe(x=Example4b[ , c("cognition","age","grip","sexMW")])

print("R Descriptive Statistics for Categorical Variables")
prop.table(table(x=Example4b$sexMW,useNA="ifany"))
prop.table(table(x=Example4b$demgroup,useNA="ifany"))

txtComment(" ") # insert blank space in output

print("R Pearson Correlation Matrix with P-values using rcorr from Hmisc package")
print("Must convert data frame to matrix to use rcorr")
rcorr(x=as.matrix(Example4b[,c("cognition","age","grip","sexMW")]), type="pearson")

txtComment(" ") # insert blank space in output

print("R Linear + Quadratic Age (0=85) Predicting Cognition")
ModelQAge = lm(data=Example4b, formula=cognition~1+age85+I(age85^2))
supernova(ModelQAge)            # supernova prints sums of squares and residual variance
SummaryCI(ModelQAge, level=.95) # custom function to add CIs to fixed effects table

txtComment(" ") # insert blank space in output

print("R Linear + Quadratic Grip (0=9) Predicting Cognition")
ModelQGrip = lm(data=Example4b, formula=cognition~1+grip9+I(grip9^2))
supernova(ModelQGrip)            # supernova prints sums of squares and residual variance
SummaryCI(ModelQGrip, level=.95) # custom function to add CIs to fixed effects table

txtComment(" ") # insert blank space in output

print("R 3-Category Dementia Group (Ref=None) Predicting Cognition")
ModelDem = lm(data=Example4b, formula=cognition~1+demNF+demNC)
supernova(ModelDem)            # supernova prints sums of squares and residual variance
SummaryCI(ModelDem, level=.95) # custom function to add CIs to fixed effects table

print("R Ask for predicted cognition per group and group differences")
print("In number lists below, values are multiplier for each fixed effect IN ORDER")
glhtModelDem = glht(model=ModelDem, linfct=rbind(
  "Cognition Mean: None"         = c(1, 0, 0),  
  "Cognition Mean: Future"       = c(1, 1, 0), 
  "Cognition Mean: Current"      = c(1, 0, 1), 
  "Mean Diff: None vs Future"    = c(0, 1, 0),
  "Mean Diff: None vs Current"   = c(0, 0, 1),
  "Mean Diff: Future vs Current" = c(0,-1, 1)))
glhtSummaryCI(glhtModelDem, level=.95)  # custom function to add CIs to glht output table
glhtEffectSizes(glhtObject=glhtModelDem, modelObject=ModelDem, 
                level=.95) # custom function to compute glht effect sizes 

txtComment(" ") # insert blank space in output


################################################################################
#####           SEQUENTIAL MODEL ANALYSES FOR UNIQUE EFFECTS              #####
################################################################################

print("R Linear Age (0=85) Predicting Cognition")
ModelAge = lm(data=Example4b, formula=cognition~1+age85)
supernova(ModelAge)            # supernova prints sums of squares and residual variance
SummaryCI(ModelAge, level=.95) # custom function to add CIs to fixed effects table

txtComment(" ") # insert blank space in output

print("R Add Linear Grip Strength (0=9) Predicting Cognition")
ModelAgeGrip = lm(data=Example4b, formula=cognition~1+age85+grip9)
supernova(ModelAgeGrip)            # supernova prints sums of squares and residual variance
SummaryCI(ModelAgeGrip, level=.95) # custom function to add CIs to fixed effects table

txtComment(" ") # insert blank space in output

print("R Add Binary Sex (0=M, 1=W) Predicting Cognition")
ModelAgeGripSex = lm(data=Example4b, formula=cognition~1+age85+grip9+sexMW)
supernova(ModelAgeGripSex)            # supernova prints sums of squares and residual variance
SummaryCI(ModelAgeGripSex, level=.95) # custom function to add CIs to fixed effects table

txtComment(" ") # insert blank space in output

###############################################################################################
print("R Combined Model Predicting Cognition")
ModelCombined = lm(data=Example4b, formula=cognition~1+age85+grip9+sexMW+demNF+demNC)
supernova(ModelCombined)            # supernova prints sums of squares and residual variance
SummaryCI(ModelCombined, level=.95) # custom function to add CIs to fixed effects table

# Get F-test and effect sizes for fixed slopes of interest using custom function
R2changeF(ReducedModel=ModelAge,        FullModel=ModelAgeGrip,    PredName="Grip")
R2changeF(ReducedModel=ModelAgeGrip,    FullModel=ModelAgeGripSex, PredName="Sex")
R2changeF(ReducedModel=ModelAgeGripSex, FullModel=ModelCombined,   PredName="Dementia")

### How to get F-test and change in R2 for multiple fixed slopes more generally ###

# Fit model without fixed slopes of interest (age85 and sexMW for demographics here)
ModelNoAgeSex = lm(data=Example4b, formula=cognition~1+grip9+demNF+demNC)
# Get F-test and effect sizes for fixed slopes of interest using custom function
R2changeF(ReducedModel=ModelNoAgeSex, FullModel=ModelCombined, PredName="Demographics")

print("Effect Sizes for Model Fixed Effects")
FixedEffectSizes(ModelCombined)  # custom function to add effect sizes for fixed slopes

print("R Ask for missing model-implied group difference")
glhtModelCombined = glht(model=ModelCombined, linfct=rbind("Future vs Current" = c(0,0,0,0,-1,1)))
glhtSummaryCI(glhtModelCombined, level=.95)  # custom function to add CIs to glht output table
glhtEffectSizes(glhtObject=glhtModelCombined, modelObject=ModelCombined, 
                level=.95) # custom function to compute glht effect sizes 

# Close output text file
txtStop()


